# -*- coding: utf-8 -*-
#
#  This file is part of django-mysql-manager.
#
#  django-mysql-manager is a Django based management interface for MySQL users and databases.
#
#  Development Web Site:
#    - http://www.codetrax.org/projects/django-mysql-manager
#  Public Source Code Repository:
#    - https://source.codetrax.org/hgroot/django-mysql-manager
#
#  Copyright 2011 George Notaras <gnot [at] g-loaded.eu>
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

try:
    from django.apps import apps
    get_model = apps.get_model
except ImportError:
    from django.db.models.loading import cache
    get_model = cache.get_model
from django.contrib import messages


def reset_database(modeladmin, request, queryset):
    MySQLDatabase = get_model('mysql_manager', 'MySQLDatabase')
    n = queryset.count()
    
    for db_obj in queryset:
        
        # Store the database info in variables
        db_name = db_obj.name
        db_user_obj = db_obj.user
        db_charset_name = db_obj.charset_name
        db_collation_name = db_obj.collation_name
        db_created_by_obj = db_obj.created_by
        
        # Delete the current database
        db_obj.delete()
        
        # Create a new database object with the stored info
        new_db_obj = MySQLDatabase(
            name = db_name,
            user = db_user_obj,
            charset_name = db_charset_name,
            collation_name = db_collation_name,
            created_by = db_created_by_obj
            )
        # First create the MySQL database on the MySQL server
        MySQLDatabase.objects.create_database(db_name, db_charset_name, db_collation_name)
        # Grant privileges on the user
        MySQLDatabase.objects.grant_privileges(db_name, db_user_obj.name)
        # Now save the Django object
        new_db_obj.save()
        
    if n:
        messages.info(request, 'Successful reset of %d database(s)' % n)
reset_database.short_description = "Reset database [CAUTION: Data Loss]"




# -*- coding: utf-8 -*-
#
#  This file is part of django-mysql-manager.
#
#  django-mysql-manager is a Django based management interface for MySQL users and databases.
#
#  Development Web Site:
#    - http://www.codetrax.org/projects/django-mysql-manager
#  Public Source Code Repository:
#    - https://source.codetrax.org/hgroot/django-mysql-manager
#
#  Copyright 2011 George Notaras <gnot [at] g-loaded.eu>
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

from django.conf import settings


_MYMANAGER_FORBIDDEN_USER_NAMES = (
    'mysql',
    'admin',
    'administrator',
    'root',
    'sys',
    'system',
    )
MYMANAGER_FORBIDDEN_USER_NAMES = getattr(settings, 'MYMANAGER_FORBIDDEN_USER_NAMES', _MYMANAGER_FORBIDDEN_USER_NAMES)

_MYMANAGER_FORBIDDEN_DATABASE_NAMES = (
    'mysql',
    'test',
    )
MYMANAGER_FORBIDDEN_DATABASE_NAMES = getattr(settings, 'MYMANAGER_FORBIDDEN_DATABASE_NAMES', _MYMANAGER_FORBIDDEN_DATABASE_NAMES)

_MYMANAGER_DEFAULT_DB_PRIVILEGES = [
        'CREATE',
        'DROP',
        'REFERENCES',
        'ALTER',
        'DELETE',
        'INDEX',
        'INSERT',
        'SELECT',
        'UPDATE',
        'CREATE TEMPORARY TABLES',
        'LOCK TABLES',
        'CREATE VIEW',
        'SHOW VIEW',
        'ALTER ROUTINE',
        'CREATE ROUTINE',
        'EXECUTE',
    #    'FILE',
    ]
MYMANAGER_DEFAULT_DB_PRIVILEGES = getattr(settings, 'MYMANAGER_DEFAULT_DB_PRIVILEGES', _MYMANAGER_DEFAULT_DB_PRIVILEGES)




# -*- coding: utf-8 -*-
#
#  This file is part of django-primary-blog.
#
#  django-primary-blog is a blogging system based on the Django framework.
#
#  Development Web Site:
#    - http://www.codetrax.org/projects/django-primary-blog
#  Public Source Code Repository:
#    - https://source.codetrax.org/hgroot/django-primary-blog
#
#  Copyright 2010 George Notaras <gnot [at] g-loaded.eu>
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

#from optparse import make_option
from django.core.management.base import BaseCommand

from mysql_manager import settings


class Command(BaseCommand):
    
    help = 'Generate an SQL statement that creates the Administrator MySQL user.'
    #args = '[appname ...]'
    requires_model_validation = False
    
    def handle(self, *test_labels, **options):
        privileges = settings.MYMANAGER_DEFAULT_DB_PRIVILEGES
        privileges.extend(['CREATE USER', 'RELOAD'])
        print "CREATE USER 'administrator'@'localhost' IDENTIFIED BY '1234';"
        print "GRANT %s ON *.* TO 'administrator'@'localhost' WITH GRANT OPTION;" % ','.join(privileges)
        




# -*- coding: utf-8 -*-
#
#  This file is part of django-mysql-manager.
#
#  django-mysql-manager is a Django based management interface for MySQL users and databases.
#
#  Development Web Site:
#    - http://www.codetrax.org/projects/django-mysql-manager
#  Public Source Code Repository:
#    - https://source.codetrax.org/hgroot/django-mysql-manager
#
#  Copyright 2011 George Notaras <gnot [at] g-loaded.eu>
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

from django.db import models
from django.db.models import signals
from django.utils.translation import ugettext_lazy as _

from mysql_manager import signal_cb
from mysql_manager import managers



class MySQLUser(models.Model):
    
    name = models.SlugField(max_length=55, db_index=True, unique=True, verbose_name=_('name'), help_text='''Enter a name for the MySQL user''')
    is_active = models.BooleanField(default=True, db_index=True, verbose_name='active', help_text='''Mark the user as active or not. A user that is marked as inactive cannot login to the PostgreSQL cluster.''')
    
    max_queries_per_hour = models.PositiveIntegerField(default=0, verbose_name='max queries per hour', help_text="""Restrict the number of queries permitted to this account during any given one-hour period. If this is set to 0 (the default), this means that there is no limitation for the account.""")
    max_updates_per_hour = models.PositiveIntegerField(default=0, verbose_name='max updates per hour', help_text="""Restrict the number of updates permitted to this account during any given one-hour period. If this is set to 0 (the default), this means that there is no limitation for the account.""")
    max_connections_per_hour = models.PositiveIntegerField(default=0, verbose_name='max connections per hour', help_text="""Restrict the number of connections permitted to this account during any given one-hour period. If this is set to 0 (the default), this means that there is no limitation for the account.""")
    max_user_connections= models.PositiveIntegerField(default=0, verbose_name='max user connections', help_text="""Restricts the maximum number of simultaneous connections to the server by the account. A nonzero value specifies the limit for the account explicitly. If value is 0 (the default), the server determines the number of simultaneous connections for the account from the global value of the ``max_user_connections`` system variable. If ``max_user_connections`` is also zero, there is no limit for the account.""")
    
    date_created = models.DateTimeField(verbose_name='created on', auto_now_add=True)
    date_modified = models.DateTimeField(verbose_name='last modified on', auto_now=True)
    created_by = models.ForeignKey('auth.User', related_name='%(app_label)s_%(class)s_created_by')
    
    objects = managers.MySQLUserManager()
    
    class Meta:
        verbose_name = 'MySQL User'
        verbose_name_plural = 'MySQL Users'
        ordering = ('name',)
    
    def __unicode__(self):
        return self.name

signals.pre_delete.connect(signal_cb.drop_user, sender=MySQLUser)



class MySQLDatabase(models.Model):
    
    name = models.SlugField(max_length=100, db_index=True, unique=True, verbose_name='name', help_text='''Enter a name for the PostgreSQL database.''')
    user = models.ForeignKey('mysql_manager.MySQLUser', related_name='%(app_label)s_%(class)s_user', on_delete=models.SET_NULL, blank=True, null=True, verbose_name=_('user'), help_text='''Select the user that owns the database.''')
    charset_name = models.CharField(max_length=30, default='utf8', verbose_name='character set', help_text='''Enter the character set for this database.''')
    collation_name = models.CharField(max_length=30, default='utf8_unicode_ci', verbose_name='collation', help_text='''Enter the collation for this database.''')
    
    date_created = models.DateTimeField(verbose_name='created on', auto_now_add=True)
    date_modified = models.DateTimeField(verbose_name='last modified on', auto_now=True)
    created_by = models.ForeignKey('auth.User', related_name='%(app_label)s_%(class)s_created_by')
    
    objects = managers.MySQLDatabaseManager()
    
    class Meta:
        verbose_name = 'MySQL Database'
        verbose_name_plural = 'MySQL Databases'
        ordering = ('name',)

    def __unicode__(self):
        return self.name

signals.pre_delete.connect(signal_cb.drop_database, sender=MySQLDatabase)





# -*- coding: utf-8 -*-
#
#  This file is part of django-mysql-manager.
#
#  django-mysql-manager is a Django based management interface for MySQL users and databases.
#
#  Development Web Site:
#    - http://www.codetrax.org/projects/django-mysql-manager
#  Public Source Code Repository:
#    - https://source.codetrax.org/hgroot/django-mysql-manager
#
#  Copyright 2011 George Notaras <gnot [at] g-loaded.eu>
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

from django.contrib import admin
try:
    from django.apps import apps
    get_model = apps.get_model
except ImportError:
    from django.db.models.loading import cache
    get_model = cache.get_model

from mysql_manager.forms import MySQLUserModelForm, MySQLDatabaseModelForm
from mysql_manager.actions import reset_database


class MySQLUserAdmin(admin.ModelAdmin):

    form = MySQLUserModelForm
    list_display = ('name', 'date_created', 'max_queries_per_hour',
        'max_updates_per_hour', 'max_connections_per_hour', 'max_user_connections')
    search_fields = ('name', )
    
    def get_form(self, request, obj=None, **kwargs):
        if obj is None:     # This is the add form
            self.fields = ['name', 'password1', 'password2',
                'max_queries_per_hour', 'max_updates_per_hour',
                'max_connections_per_hour', 'max_user_connections']
        else:               # This is the change form
            self.fields = ['name', 'password1', 'password2',
                'date_created', 'date_modified',
                'max_queries_per_hour', 'max_updates_per_hour',
                'max_connections_per_hour', 'max_user_connections']
            if request.user.is_superuser:
                self.fields.append('created_by')
        return super(MySQLUserAdmin, self).get_form(request, obj, **kwargs)
    
    def get_readonly_fields(self, request, obj=None):
        if obj is None:     # This is the add form
            self.readonly_fields = []
        else:               # This is the change form
            self.readonly_fields = ['name', 'date_created', 'date_modified']
            if request.user.is_superuser:
                self.readonly_fields.append('created_by')
        return super(MySQLUserAdmin, self).get_readonly_fields(request, obj)
    
    def get_queryset(self, request):
        qs = super(MySQLUserAdmin, self).get_queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.filter(created_by=request.user)
    
    def save_model(self, request, obj, form, change):
        MySQLUser = get_model('mysql_manager', 'MySQLUser')
        password = form.cleaned_data.get('password1')
        
        if not change:  # User creation form
            
            assert password != '', 'A password is mandatory for new accounts'
            
            # The ``created_by`` attribute is set once at creation time
            obj.created_by = request.user
            
            MySQLUser.objects.create_user(obj.name, password)
            MySQLUser.objects.set_limits(obj.name, obj.max_queries_per_hour,
                obj.max_updates_per_hour, obj.max_connections_per_hour, obj.max_user_connections)
            
        else:   # This is the change form
            
            if 'max_queries_per_hour' in form.changed_data or \
                    'max_updates_per_hour' in form.changed_data or \
                    'max_connections_per_hour' in form.changed_data or \
                    'max_user_connections' in form.changed_data:
                MySQLUser.objects.set_limits(obj.name, obj.max_queries_per_hour,
                    obj.max_updates_per_hour, obj.max_connections_per_hour, obj.max_user_connections)
            
            if password:
                MySQLUser.objects.change_password(obj.name, password)
 
        # Save the model
        obj.save()

admin.site.register(get_model('mysql_manager', 'MySQLUser'), MySQLUserAdmin)



class MySQLDatabaseAdmin(admin.ModelAdmin):

    form = MySQLDatabaseModelForm
    list_display = ('name', 'user', 'charset_name', 'collation_name', 'date_created')
    
    search_fields = ('name', 'owner__name')
    
    actions = [reset_database, ]
    
    def get_form(self, request, obj=None, **kwargs):
        if obj is None:     # This is the add form
            self.fields = ['name', 'user', 'charset_name', 'collation_name']
        else:               # This is the change form
            self.fields = ['name', 'user', 'charset_name', 'collation_name', 'date_created', 'date_modified']
            if request.user.is_superuser:
                self.fields.append('created_by')
        return super(MySQLDatabaseAdmin, self).get_form(request, obj, **kwargs)
    
    def get_readonly_fields(self, request, obj=None):
        if obj is None:     # This is the add form
            self.readonly_fields = []
        else:               # This is the change form
            self.readonly_fields = ['name', 'date_created', 'date_modified']
            if request.user.is_superuser:
                self.readonly_fields.append('created_by')
        return super(MySQLDatabaseAdmin, self).get_readonly_fields(request, obj)
    
    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        MySQLUser = get_model('mysql_manager', 'MySQLUser')
        if db_field.name == 'user':
            if not request.user.is_superuser:
                kwargs['queryset'] = MySQLUser.objects.filter(created_by=request.user)
        return super(MySQLDatabaseAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)
    
    def get_queryset(self, request):
        qs = super(MySQLDatabaseAdmin, self).get_queryset(request)
        if request.user.is_superuser:
            return qs
        return qs.filter(created_by=request.user)
    
    def save_model(self, request, obj, form, change):
        MySQLDatabase = get_model('mysql_manager', 'MySQLDatabase')
        
        if not change:  # Database creation form
            
            # The ``created_by`` attribute is set once at creation time
            obj.created_by = request.user
            
            MySQLDatabase.objects.create_database(obj.name, obj.charset_name, obj.collation_name)
            
            if obj.user:
                MySQLDatabase.objects.grant_privileges(obj.name, obj.user.name)
        
        else:   # This is the change form
            
            if 'charset_name' in form.changed_data or 'collation_name' in form.changed_data:
                MySQLDatabase.objects.change_specification(obj.name, obj.charset_name, obj.collation_name)
            
            if 'user' in form.changed_data:
                # First revoke all privileges on the old database.
                old_user_id = form.initial.get('user', None)
                if old_user_id is not None:
                    # old_user_id can be none in cases where it has never been set before.
                    MySQLUser = get_model('mysql_manager', 'MySQLUser')
                    old_user = MySQLUser.objects.get(id__exact=old_user_id)
                    MySQLDatabase.objects.revoke_privileges(obj.name, old_user.name)

            if obj.user:
                # Now grant all privileges to the new owner
                MySQLDatabase.objects.grant_privileges(obj.name, obj.user.name)
                
        # Save the model
        obj.save()

admin.site.register(get_model('mysql_manager', 'MySQLDatabase'), MySQLDatabaseAdmin)




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.deletion
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='MySQLDatabase',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.SlugField(help_text=b'Enter a name for the PostgreSQL database.', unique=True, max_length=100, verbose_name=b'name')),
                ('charset_name', models.CharField(default=b'utf8', help_text=b'Enter the character set for this database.', max_length=30, verbose_name=b'character set')),
                ('collation_name', models.CharField(default=b'utf8_unicode_ci', help_text=b'Enter the collation for this database.', max_length=30, verbose_name=b'collation')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name=b'created on')),
                ('date_modified', models.DateTimeField(auto_now=True, verbose_name=b'last modified on')),
                ('created_by', models.ForeignKey(related_name='mysql_manager_mysqldatabase_created_by', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'MySQL Database',
                'verbose_name_plural': 'MySQL Databases',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MySQLUser',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.SlugField(help_text=b'Enter a name for the MySQL user', unique=True, max_length=55, verbose_name='name')),
                ('is_active', models.BooleanField(default=True, help_text=b'Mark the user as active or not. A user that is marked as inactive cannot login to the PostgreSQL cluster.', db_index=True, verbose_name=b'active')),
                ('max_queries_per_hour', models.PositiveIntegerField(default=0, help_text=b'Restrict the number of queries permitted to this account during any given one-hour period. If this is set to 0 (the default), this means that there is no limitation for the account.', verbose_name=b'max queries per hour')),
                ('max_updates_per_hour', models.PositiveIntegerField(default=0, help_text=b'Restrict the number of updates permitted to this account during any given one-hour period. If this is set to 0 (the default), this means that there is no limitation for the account.', verbose_name=b'max updates per hour')),
                ('max_connections_per_hour', models.PositiveIntegerField(default=0, help_text=b'Restrict the number of connections permitted to this account during any given one-hour period. If this is set to 0 (the default), this means that there is no limitation for the account.', verbose_name=b'max connections per hour')),
                ('max_user_connections', models.PositiveIntegerField(default=0, help_text=b'Restricts the maximum number of simultaneous connections to the server by the account. A nonzero value specifies the limit for the account explicitly. If value is 0 (the default), the server determines the number of simultaneous connections for the account from the global value of the ``max_user_connections`` system variable. If ``max_user_connections`` is also zero, there is no limit for the account.', verbose_name=b'max user connections')),
                ('date_created', models.DateTimeField(auto_now_add=True, verbose_name=b'created on')),
                ('date_modified', models.DateTimeField(auto_now=True, verbose_name=b'last modified on')),
                ('created_by', models.ForeignKey(related_name='mysql_manager_mysqluser_created_by', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('name',),
                'verbose_name': 'MySQL User',
                'verbose_name_plural': 'MySQL Users',
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='mysqldatabase',
            name='user',
            field=models.ForeignKey(related_name='mysql_manager_mysqldatabase_user', on_delete=django.db.models.deletion.SET_NULL, blank=True, to='mysql_manager.MySQLUser', help_text=b'Select the user that owns the database.', null=True, verbose_name='user'),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
#
#  This file is part of django-mysql-manager.
#
#  django-mysql-manager is a Django based management interface for MySQL users and databases.
#
#  Development Web Site:
#    - http://www.codetrax.org/projects/django-mysql-manager
#  Public Source Code Repository:
#    - https://source.codetrax.org/hgroot/django-mysql-manager
#
#  Copyright 2011 George Notaras <gnot [at] g-loaded.eu>
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#


from django.db import models
from django.db import connections

from mysql_manager import settings


class BasePgManager(models.Manager):
    
    MYMANAGER_DB_ALIAS = 'mysql_manager_conn'
    
    DEFAULT_DB_PRIVILEGES = ','.join(settings.MYMANAGER_DEFAULT_DB_PRIVILEGES)
    
    def execute_custom_query(self, sql):
        #print sql
        connection = connections[self.MYMANAGER_DB_ALIAS]
        cursor = connection.cursor()
        cursor.execute(sql)


class MySQLUserManager(BasePgManager):
    
    def create_user(self, username, password):
        self.execute_custom_query(
            "CREATE USER '%s'@'localhost' IDENTIFIED BY '%s'" % (username, password))
        self.execute_custom_query("FLUSH PRIVILEGES")
    
    def drop_user(self, username):
        self.execute_custom_query("DROP USER '%s'@'localhost'" % username)
        self.execute_custom_query("FLUSH PRIVILEGES")
    
    def change_password(self, username, password):
        self.execute_custom_query("SET PASSWORD FOR '%s'@'localhost' = PASSWORD('%s')" % (username, password))

    def set_limits(self, username, max_queries_per_hour,
            max_updates_per_hour, max_connections_per_hour, max_user_connections):
        """
        These limits are set on a per user basisand cannot be combined with a
        specific database.
        
        Therefore we use the general ``USAGE ON *.*`` privilege here. Database
        specific privileges are set by the MySQLDatabaseManager.grant_privileges()
        method.
        """
        self.execute_custom_query("GRANT USAGE ON *.* TO '%(username)s'@'localhost' \
            WITH MAX_QUERIES_PER_HOUR %(max_queries_per_hour)s \
            MAX_UPDATES_PER_HOUR %(max_updates_per_hour)s \
            MAX_CONNECTIONS_PER_HOUR %(max_connections_per_hour)s \
            MAX_USER_CONNECTIONS %(max_user_connections)s" % {
                'username': username,
                'max_queries_per_hour': max_queries_per_hour,
                'max_updates_per_hour': max_updates_per_hour,
                'max_connections_per_hour': max_connections_per_hour,
                'max_user_connections': max_user_connections,
            }
        )

class MySQLDatabaseManager(BasePgManager):
    
    def create_database(self, database, charset, collation):
        self.execute_custom_query(
            "CREATE DATABASE %(database)s CHARACTER SET %(charset)s COLLATE %(collation)s" % {
                'database': database, 'charset': charset, 'collation': collation})
    
    def drop_database(self, database):
        self.execute_custom_query("DROP DATABASE %s" % database)
    
    def change_specification(self, database, charset, collation):
        self.execute_custom_query("ALTER DATABASE %s CHARACTER SET %s COLLATE %s" % (database, charset, collation))
    
    def grant_privileges(self, database, username):
        self.execute_custom_query("GRANT %(privileges)s ON `%(database)s`.* TO '%(username)s'@'localhost'" % {
            'privileges': self.DEFAULT_DB_PRIVILEGES,
            'database': database,
            'username': username,
        })
        self.execute_custom_query("FLUSH PRIVILEGES")
    
    def revoke_privileges(self, database, username):
        self.execute_custom_query("REVOKE %s ON `%s`.* FROM '%s'@'localhost'" % (
            self.DEFAULT_DB_PRIVILEGES, database, username))
        self.execute_custom_query("FLUSH PRIVILEGES")




# -*- coding: utf-8 -*-
#
#  This file is part of django-mysql-manager.
#
#  django-mysql-manager is a Django based management interface for MySQL users and databases.
#
#  Development Web Site:
#    - http://www.codetrax.org/projects/django-mysql-manager
#  Public Source Code Repository:
#    - https://source.codetrax.org/hgroot/django-mysql-manager
#
#  Copyright 2011 George Notaras <gnot [at] g-loaded.eu>
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

import string

from django import forms
try:
    from django.apps import apps
    get_model = apps.get_model
except ImportError:
    from django.db.models.loading import cache
    get_model = cache.get_model

from mysql_manager import settings


class MySQLUserModelForm(forms.ModelForm):

    class Meta:
        model = get_model('mysql_manager', 'MySQLUser')
        fields = '__all__'
    
    # Adds two extra password fields, which will be used for password confirmation.
    password1 = forms.CharField(label='Password', required=False, widget=forms.PasswordInput, help_text="Valid characters a-z, A-Z, 0-9 and the underscore '_'")
    password2 = forms.CharField(label='Password (confirm)', required=False, widget=forms.PasswordInput, help_text="Valid characters a-z, A-Z, 0-9 and the underscore '_'")
    
    def clean_name(self):
        name = self.cleaned_data.get('name')
        if name.lower() in settings.MYMANAGER_FORBIDDEN_USER_NAMES:
            forms.ValidationError('This name is reserved for internal use.')
        return name
    
    def clean_password1(self):
        """Assures that passord characters are: a-z, A-Z, 0-9
        """
        password1 = self.cleaned_data.get('password1')
        if password1:
            valid_chars = string.lowercase + string.uppercase + string.digits + '_'
            for character in password1:
                if character not in valid_chars:
                    self._errors['password1'] = self.error_class(['Valid characters a-z, A-Z, 0-9.'])
        return password1
        
    def clean_password2(self):
        """Cleans the content of the two extra password fields.
        """
        password1 = self.cleaned_data.get('password1')
        password2 = self.cleaned_data.get('password2')
        
        # Provided password and confirmation must match
        if password1 != password2:
            raise forms.ValidationError('The two password fields do not match.')
        
        # Password is required on new records!
        # If a password has not been set...
        if not password1 and not password2:
            if self.instance.pk is None: # ... and this is a new record
                # Attach error messages to the password fields
                self._errors['password1'] = self.error_class(['Password must be set on new records.'])
                self._errors['password2'] = self.error_class(['Password must be set on new records.'])
                
        return password2



class MySQLDatabaseModelForm(forms.ModelForm):

    class Meta:
        model = get_model('mysql_manager', 'MySQLDatabase')
        fields = '__all__'
    
    def clean_name(self):
        name = self.cleaned_data.get('name')
        if name.lower() in settings.MYMANAGER_FORBIDDEN_DATABASE_NAMES:
            forms.ValidationError('This name is reserved for internal use.')
        return name





# -*- coding: utf-8 -*-
#
#  This file is part of django-mysql-manager.
#
#  django-mysql-manager is a Django based management interface for MySQL users and databases.
#
#  Development Web Site:
#    - http://www.codetrax.org/projects/django-mysql-manager
#  Public Source Code Repository:
#    - https://source.codetrax.org/hgroot/django-mysql-manager
#
#  Copyright 2011 George Notaras <gnot [at] g-loaded.eu>
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.
#

VERSION = (0, 1, 4, 'final', 0)

def get_version():
    version = '%d.%d.%d' % (VERSION[0], VERSION[1], VERSION[2])
    return version

